#! /usr/bin/env python

import glob
import optparse as op
import os
import subprocess as sp
import sys

__usage__ = '%prog [-h] [-d DIR]'

def do_history(gb1, gb2=None):
    sys.stderr.write("DIFF: %s %s\n" % (gb1, gb2))
    command = ['neb-gbdiff', gb1]
    if gb2 is not None:
        command.append(gb2)
    sp.check_call(command)

def generate_history(dir):
    fnames = glob.glob(os.path.join(dir, "*.gbk"))
    fnames.sort()
    if len(fnames):
        do_history(fnames[0])
    for i in range(1, len(fnames)):
        do_history(fnames[i-1], fnames[i])

def options():
    return [
        op.make_option('-d', '--dir', dest='dir', default='./',
            help="Directory containing a history of Genbank files."),
    ]

def main():
    parser = op.OptionParser(usage=__usage__, option_list=options())
    opts, args = parser.parse_args()

    if len(args):
        parser.error("Unknown arguments: %s" % ' '.join(args))

    generate_history(opts.dir)

if __name__ == '__main__':
    try:
        main()
    except IOError, inst:
        # ignore BrokenPipe errors
        if inst.errno not in [32]:
            raise
    except KeyboardInterrupt:
        pass




#! /usr/bin/env python

import sys

import jsonical

import nebpack

def stdin():
    while True:
        line = sys.stdin.readline()
        if not line:
            return
        if not line.strip():
            continue
        try:
            yield jsonical.loads(line)
        except:
            sys.stderr.write("LAST LINE: %r\n" % line)
            raise

def process(pack, data):
    if data.get("type") == "keyword":
        if data.get("delete", False):
            pack.rem_keyword(data["keyword"])
        else:
            pack.add_keyword(data["keyword"], data["value"])
    elif data.get("type") == "reference":
        if data.get("delete", False):
            pack.rem_reference(data["refid"])
        else:
            pack.add_reference(data["refid"], data["value"])
    elif data.get("type") == "seqid":
        pack.add_seqid(data["seqid"].encode('utf-8'))
    elif data.get("type") == "feature":
        if data.get("delete", False):
            pack.rem_feature(data["feature"])
        else:
            desc_id = pack.desc_id(data.get("previous"))
            pack.add_feature(data["feature"], prev=desc_id)
    else:
        raise ValueError("Invalid type: %r" % data.get("type"))

def main():
    if len(sys.argv) != 2:
        print "usage: %s GIT_DIR" % sys.argv[0]
        exit(1)

    pack = nebpack.Pack(sys.argv[1])
    for obj in stdin():
        process(pack, obj)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass




#! /usr/bin/env python

import os
import optparse as op
import re
import sys
import time
import urllib

from BeautifulSoup import BeautifulSoup
import runfunc as rf

__usage__ = "%prog [-d DIR] ACC1 [ACC2 ...]"

NCBI="http://www.ncbi.nlm.nih.gov"

DATE_FMT = "%b %d %Y %I:%M %p"
FNAME_FMT = "%Y-%m-%d-%H-%m"
HREFRE = re.compile(r"[0-9A-Za-z]+:[0-9A-Za-z]+:[0-9A-Za-z]+")

def fetch_ids(acc):
    url = "%s/entrez/sutils/girevhist.cgi?val=%s" % (NCBI, acc)
    soup = BeautifulSoup(urllib.urlopen(url).read())
    ret = []
    for ln in soup.findAll("a"):
        data = ln["href"].split("?")[-1]
        if not HREFRE.match(data):
            continue
        try:
            date = time.strptime(ln.renderContents(), DATE_FMT)
        except:
            continue
        ret.append((data, date))
    return ret

def fetch(uid, date, path, count=5):
    print "Fetching: %s" % uid
    if count < 1:
        raise RuntimeError("Failed to fetch url: %s" % url)
    url = "%s/sviewer/viewer.fcgi?sendto=on&list_uids=%s"
    gbk = urllib.urlopen(url % (NCBI, uid)).read()
    if len(gbk) < 1024:
        return fetch(uid, date, path, count-1)
    if not os.path.isdir(path):
        os.mkdir(path)
    ext = "ft" if format == "table" else "gbk"
    fname = time.strftime(FNAME_FMT, date) + "." + ext
    fname = os.path.join(path, fname)
    with open(fname, 'wb') as handle:
        handle.write(gbk)


def options():
    return [
        op.make_option('-d', '--dir', dest='dir', default='./'),
    ]

def main():
    parser = op.OptionParser(usage=__usage__, option_list=options())
    opts, args = parser.parse_args()

    if len(args) < 1:
        parser.error("No accession provided.")

    if not os.path.exists(opts.dir):
        os.mkdir(opts.dir)
    
    for acc in args:
        for (uid, date) in fetch_ids(acc):
            fetch(uid, date, opts.dir)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#! /usr/bin/env python

import hashlib
import optparse as op
import os

import jsonical
import nebgb
import nebpack

__usage__ = "%prog [-p PACK] [-g GENBANK]"

def jscmp(a, b):
    a = jsonical.dumps(a, indent=4)
    b = jsonical.dumps(b, indent=4)
    if a != b:
        print a
        print b
        return False
    return True

def mkhash(data):
    return hashlib.sha1(data).hexdigest().upper()

def validate(pack, rec):
    if not jscmp(pack.get_keyword("locus"), rec.locus):
        raise ValueError("Keyword 'locus' does not match.")
    for kwname, kwval in rec.keywords.iteritems():
        if kwname == "reference":
            for r in kwval:
                rid = pack.jshash(r)
                pack.get_reference(rid) # raises error on failure
        elif not jscmp(pack.get_keyword(kwname), kwval):
            raise ValueError("Keyword %r does not match.")
    active_ids = pack.active_ids.copy()
    count = 0
    for feat in rec.features:
        count += 1
        if count % 100 == 0:
            print "%d of %d" % (count, len(rec.features))
        props = feat["properties"]
        if "translation" in props:
            props["translation"] = mkhash(props["translation"]) 
        descid = pack.desc_id(feat)
        pf = pack.get_feature(descid)
        pf.pop("deleted", None)
        pf.pop("previous", None)
        if not jscmp(pf, feat):
            raise ValueError("Features don't match.")
        active_ids.pop(descid)
        active_ids.pop(pack.get_feature_id(descid))
    if len(active_ids):
        for aid, aidv in active_ids.iteritems():
            print "%s -> %s" % (aid, aidv)

def options():
    return [
        op.make_option('-p', '--pack', dest='pack', default='genome.pack',
            help="Genome pack to validate."),
        op.make_option('-g', '--gb', dest='gb', default='genome.gb',
            help="Genbank to validate the pack against.")
    ]

def main():
    parser = op.OptionParser(usage=__usage__, option_list=options())
    opts, args = parser.parse_args()
    
    if len(args):
        parser.error("Unknown arguments: %s" % ' '.join(args))

    if not os.path.isdir(opts.pack):
        parser.error("Pack path is not a directory: %s" % opts.pack)
    
    if not os.path.isfile(opts.gb):
        parser.error("Genbank path is not a file: %s" % opts.gb)
    
    pack = nebpack.Pack(opts.pack)
    rec = nebgb.parse_file(opts.gb).next()
    validate(pack, rec)

if __name__ == '__main__':
    main()



#
# Copyright 2009 New England Biolabs <davisp@neb.com>
#
# This file is part of the nebgbhist package released under the MIT license.
#

import os
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup

base_dir = os.path.dirname(__file__)
long_desc = open(os.path.join(base_dir, "README.md")).read()

setup(
    name = "nebgbhist",
    description = "Annotation histories from Genbank files.",
    long_description = long_desc,
    author = "Paul Joseph Davis",
    author_email = "davisp@neb.com",
    url = "http://github.com/davisp/nebgbhist",
    version = "0.0.3",
    license = "MIT",
    keywords = "bioinformatics Genbank annotation history",
    platforms = ["any"],
    zip_safe = False,

    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],

    packages = ["nebgbhist"],
    
    entry_points = {
        'console_scripts': [
            'neb-diff-apply = nebgbhist.apply:main',
            'neb-rev-fetch = nebgbhist.fetch:main',
            'neb-gbhist = nebgbhist.history:main',
            'neb-validate-history = nebgbhist.validate:main'
        ]
    },

    setup_requires = ["setuptools>=0.6c8"],
    tests_require = ["nose>=0.10.0"],
    install_requires = [
        "nebgb>=0.1.2",
        "jsonical>=0.0.1",
        "nebgbdiff>=0.0.3",
        "nebpack>=0.0.3",
        "BeautifulSoup>=3.1.0.0",
    ],

    test_suite = "nose.collector"
)
